#include "LoadShaders.h"

LoadShaders::LoadShaders()
{
}


LoadShaders::~LoadShaders()
{
}

GLuint LoadShaders::loadShader(const char* vertex_file_path, const char* fragment_file_path)
{
	//create vertex & fragment shader
	GLuint vertexShaderID = glCreateShader(GL_VERTEX_SHADER);
	GLuint fragmentShaderID = glCreateShader(GL_FRAGMENT_SHADER);

	//read vertex shader code from file
	std::string vertexShaderCode;
	std::ifstream VertexShaderStream(vertex_file_path, std::ios::in);
	if (VertexShaderStream.is_open())
	{
		std::string line = "";
		while (getline(VertexShaderStream, line))
		{
			vertexShaderCode += "\n" + line;
		};
		VertexShaderStream.close();
	}
	else
	{
		std::cout << "Error: Impossible to open vertex shader file." << std::endl;
	}

	//read fragment shader code from file
	std::string fragmentShaderCode;
	std::ifstream FragmentShaderStream(fragment_file_path, std::ios::in);
	if (FragmentShaderStream.is_open())
	{
		std::string line = "";
		while (getline(FragmentShaderStream, line))
		{
			fragmentShaderCode += "\n" + line;
		};
		FragmentShaderStream.close();
	}
	else
	{
		std::cout << "Error: Impossible to open fragment shader file." << std::endl;
	}


	GLint result = GL_FALSE;
	int infoLogLength;

	//compile vertex shader
	std::cout << "start compiling vertex shader: " << vertex_file_path << std::endl;
	char const* vertexSourcePointer = vertexShaderCode.c_str();
	glShaderSource(vertexShaderID, 1, &vertexSourcePointer, NULL);
	glCompileShader(vertexShaderID);

	//check vertex shader
	glGetShaderiv(vertexShaderID, GL_COMPILE_STATUS, &result);
	glGetShaderiv(vertexShaderID, GL_INFO_LOG_LENGTH, &infoLogLength);
	if (infoLogLength > 0)
	{
		std::vector<char> VertexShaderErrorMessage(infoLogLength + 1);
		glGetShaderInfoLog(vertexShaderID, infoLogLength, NULL, &VertexShaderErrorMessage[0]);
		std::cout << "" << &VertexShaderErrorMessage[0] << std::endl;
	}

	//compile fragment shader
	std::cout << "start compiling fragment shader: " << vertex_file_path << std::endl;
	char const* fragmentSourcePointer = fragmentShaderCode.c_str();
	glShaderSource(fragmentShaderID, 1, &fragmentSourcePointer, NULL);
	glCompileShader(fragmentShaderID);

	//check fragment shader
	glGetShaderiv(fragmentShaderID, GL_COMPILE_STATUS, &result);
	glGetShaderiv(fragmentShaderID, GL_INFO_LOG_LENGTH, &infoLogLength);
	if (infoLogLength > 0){
		std::vector<char> FragmentShaderErrorMessage(infoLogLength + 1);
		glGetShaderInfoLog(fragmentShaderID, infoLogLength, NULL, &FragmentShaderErrorMessage[0]);
		std::cout << "" << &FragmentShaderErrorMessage[0] << std::endl;
	}


	//link the program bzw. generating program object
	std::cout << "Linking the program" << std::endl;
	programID = glCreateProgram();
	glAttachShader(programID, vertexShaderID);
	glAttachShader(programID, fragmentShaderID);
	glLinkProgram(programID);

	//check the program
	glGetProgramiv(programID, GL_LINK_STATUS, &result);
	glGetProgramiv(programID, GL_INFO_LOG_LENGTH, &infoLogLength);
	if (infoLogLength > 0)
	{
		std::vector<char> ProgramErrorMessage(infoLogLength + 1);
		glGetProgramInfoLog(programID, infoLogLength, NULL, &ProgramErrorMessage[0]);
		std::cout << "" << &ProgramErrorMessage[0] << std::endl;
	}

	glDetachShader(programID, vertexShaderID);
	glDetachShader(programID, fragmentShaderID);

	glDeleteShader(vertexShaderID);
	glDeleteShader(fragmentShaderID);

	//enable depthtest
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);

	glEnable(GL_CULL_FACE);//enable backface-culling

	return programID;
}

void LoadShaders::useShader()
{
	glUseProgram(programID);
}

GLuint LoadShaders::getProgramID()
{	
	return programID;
}