#include "ParticleShader.h"

static GLfloat* particles_position_size_data = new GLfloat[MaxParticles * 4];
static GLubyte* particles_color_data = new GLubyte[MaxParticles * 4];

// particlesPerFrame <= MaxParticles
ParticleShader::ParticleShader(Camera *camera, LoadShaders *particleShader){
	this->camera = camera;
	this->particleShader = particleShader;

	createBuffers();
}

ParticleShader::~ParticleShader(){

}

// create Buffer for Particles
void ParticleShader::createBuffers(){
	
	glGenVertexArrays(1, &VertexArrayID);
	glBindVertexArray(VertexArrayID);

	//std::cout << "createBuffers" << std::endl;
	
	// VBO
	static const GLfloat vertex_buffer_data[] = {
		-0.1f, -0.1f, 0.0f,
		0.1f, -0.1f, 0.0f,
		-0.1f, 0.1f, 0.0f,
		0.1f, 0.1f, 0.0f,
	};

	// vertex_buffer
	glGenBuffers(1, &vertex_buffer);
	glBindBuffer(GL_ARRAY_BUFFER, vertex_buffer);
	glBufferData(GL_ARRAY_BUFFER, sizeof(vertex_buffer_data), vertex_buffer_data, GL_STATIC_DRAW);

	// The VBO containing the positions and sizes of the particles
	// particles_position_buffer
	glGenBuffers(1, &particles_position_buffer);
	glBindBuffer(GL_ARRAY_BUFFER, particles_position_buffer);
	
	glBufferData(GL_ARRAY_BUFFER, MaxParticles * 4 * sizeof(GLfloat), NULL, GL_STREAM_DRAW);

	// The VBO containing the colors of the particles
	// particles_color_buffer
	glGenBuffers(1, &particles_color_buffer);
	glBindBuffer(GL_ARRAY_BUFFER, particles_color_buffer);
	
	glBufferData(GL_ARRAY_BUFFER, MaxParticles * 4 * sizeof(GLubyte), NULL, GL_STREAM_DRAW);

	for (int i = 0; i<MaxParticles; i++){
		particlesContainer[i].life = -1.0f;
		particlesContainer[i].cameraDistance = -1.0f;
	}
}

// bind the Buffers
void ParticleShader::updateBuffers(){

	//std::cout << "updateBuffers" << std::endl;

	glBindBuffer(GL_ARRAY_BUFFER, particles_position_buffer);
	glBufferData(GL_ARRAY_BUFFER, MaxParticles * 4 * sizeof(GLfloat), NULL, GL_STREAM_DRAW); 
	glBufferSubData(GL_ARRAY_BUFFER, 0, particlesCount * sizeof(GLfloat) * 4, particles_position_size_data);

	glBindBuffer(GL_ARRAY_BUFFER, particles_color_buffer);
	glBufferData(GL_ARRAY_BUFFER, MaxParticles * 4 * sizeof(GLubyte), NULL, GL_STREAM_DRAW); 
	glBufferSubData(GL_ARRAY_BUFFER, 0, particlesCount * sizeof(GLubyte) * 4, particles_color_data);

	glBindVertexArray(VertexArrayID);
	// 1) vertices
	glEnableVertexAttribArray(0);
	glBindBuffer(GL_ARRAY_BUFFER, vertex_buffer);
	glVertexAttribPointer(
		0, // attribute. No particular reason for 0, but must match the layout in the shader.
		3, // size
		GL_FLOAT, // type
		GL_FALSE, // normalized?
		0, // stride
		(void*)0 // array buffer offset
		);

	// 2) positions of particles' centers
	glEnableVertexAttribArray(1);
	glBindBuffer(GL_ARRAY_BUFFER, particles_position_buffer);
	glVertexAttribPointer(
		1, // attribute. No particular reason for 1, but must match the layout in the shader.
		4, // size : x + y + z + size => 4
		GL_FLOAT, // type
		GL_FALSE, // normalized?
		0, // stride
		(void*)0 // array buffer offset
		);

	// 3) particles' colors
	glEnableVertexAttribArray(2);
	glBindBuffer(GL_ARRAY_BUFFER, particles_color_buffer);
	glVertexAttribPointer(
		2, // attribute. No particular reason for 1, but must match the layout in the shader.
		4, // size : r + g + b + a => 4
		GL_UNSIGNED_BYTE, // type
		GL_TRUE, // normalized? *** YES, this means that the unsigned char[4] will be accessible with a vec4 (floats) in the shader ***
		0, // stride
		(void*)0 // array buffer offset
		);

}

// draw the Instances
void ParticleShader::renderParticles(){
	
	glVertexAttribDivisor(0, 0); // particles vertices : always reuse the same 4 vertices -> 0
	std::cout << "glVertexAttribDivisor(0, 0)" << std::endl;
	glVertexAttribDivisor(1, 1); // positions : one per quad (its center) -> 1
	std::cout << "glVertexAttribDivisor(1, 1)" << std::endl;
	glVertexAttribDivisor(2, 1); // color : one per quad -> 1
	std::cout << "glVertexAttribDivisor(2, 1)" << std::endl;

	std::cout << "particlesCount " << particlesCount << std::endl;
	std::cout << "openGL Error " << glGetError()<<std::endl;

	glDrawArraysInstanced(GL_TRIANGLE_STRIP, 0, 4, particlesCount);
	std::cout << "openGL Error " << glGetError() << std::endl;
	
	std::cout << "glDrawArraysInstanced(GL_TRIANGLE_STRIP, 0, 4, particlesCount)" << std::endl;


	glDisableVertexAttribArray(0);
	std::cout << "	glDisableVertexAttribArray(0);" << std::endl;
	glDisableVertexAttribArray(1);
	std::cout << "glDisableVertexAttribArray(1);" << std::endl;
	glDisableVertexAttribArray(2);
	std::cout << "glDisableVertexAttribArray(2);" << std::endl;

}

// Finds a Particle in particlesContainer which isn't used yet. (life < 0)
int ParticleShader::findUnusedParticle(){

	for (int i = lastUsedParticle; i<MaxParticles; i++){
		if (particlesContainer[i].life < 0){
			lastUsedParticle = i;
			return i;
		}
	}

	for (int i = 0; i<lastUsedParticle; i++){
		if (particlesContainer[i].life < 0){
			lastUsedParticle = i;
			return i;
		}
	}

	return 0; // All particles are taken, override the first one
}

void ParticleShader::setParticlesPerFrame(int newParticlesPerFrame){
	particlesPerFrame = (int)(timer->getDeltaT() * newParticlesPerFrame);
	//std::cout << "particlesPerFrame: " << particlesPerFrame << std::endl;

	//if (particlesPerFrame > (int)(0.016f * newParticlesPerFrame))
		//particlesPerFrame = (int)(0.016f * newParticlesPerFrame);
	std::cout << "particlesPerFrame: " << particlesPerFrame << std::endl;
}

void ParticleShader::loadParticlesToGpu(){
	particlesCount = 0;

	for (int i = 0; i<MaxParticles; i++){

		Particle& p = particlesContainer[i]; // shortcut

		if (p.life > 0.0f){

			// Decrease life
			p.life -= (timer->getDeltaT());
			if (p.life > 0.0f){

				// Simulate simple physics : gravity only, no collisions
				p.speed += vec3(0.0f, -9.81f, 0.0f) * (float)timer->getDeltaT() * 0.4f;
				p.pos += p.speed * (float)(timer->getDeltaT());
				p.cameraDistance = length2(p.pos - camera->getPosition());
				//particlesContainer[i].pos += glm::vec3(0.0f,10.0f, 0.0f) * (float)timer.getDeltaT();

				// Fill the GPU buffer
				particles_position_size_data[4 * particlesCount + 0] = p.pos.x;
				particles_position_size_data[4 * particlesCount + 1] = p.pos.y;
				particles_position_size_data[4 * particlesCount + 2] = p.pos.z;

				particles_position_size_data[4 * particlesCount + 3] = p.size;

				particles_color_data[4 * particlesCount + 0] = p.r;
				particles_color_data[4 * particlesCount + 1] = p.g;
				particles_color_data[4 * particlesCount + 2] = p.b;
				particles_color_data[4 * particlesCount + 3] = p.a;

			}
			else{
				// Particles that just died will be put at the end of the buffer in SortParticles();
				p.cameraDistance = -1.0f;
			}

			particlesCount++;

		}
	}

}

void ParticleShader::sortParticles(){
	std::sort(&particlesContainer[0], &particlesContainer[MaxParticles]);
}

void ParticleShader::setUpParticles(Camera *camera, Mouse *mouse, Timer *timer, int particlesPerFrame){
	this->timer = timer;
	this->mouse = mouse;
	this->camera = camera;
	
	setParticlesPerFrame(particlesPerFrame);

	for (int i = 0; i<particlesPerFrame; i++){
		int particleIndex = findUnusedParticle();
		particlesContainer[particleIndex].life = 5.0f; // This particle will live 4 seconds.
		particlesContainer[particleIndex].pos = vec3(mouse->getPosition().x, mouse->getPosition().y, mouse->getPosition().z);

		float spread = 1.5f;
		glm::vec3 maindir = glm::vec3(0.0f, 2.5f, 0.0f);

		glm::vec3 randomdir = glm::vec3(
			(rand() % 200 - 100.0f) / 100.0f,
			-(rand() % 200 - 100.0f) / 100.0f,
			(rand() % 200 - 100.0f) / 100.0f
			);

		particlesContainer[particleIndex].speed = maindir + randomdir*spread;

		// Very bad way to generate a random color
		particlesContainer[particleIndex].r = rand() % 220;
		particlesContainer[particleIndex].g = rand() % 145 + 5;
		particlesContainer[particleIndex].b = rand() % 20;
		particlesContainer[particleIndex].a = (rand() % 156) +100 / 2;

		particlesContainer[particleIndex].size = (rand() % 1000) / 2000.0f + 0.1f;

	}

	loadParticlesToGpu();
	sortParticles();

	// Use our shader
	particleShader->useShader();

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	mat4 ViewProjectionMatrix = camera->getProjectionMatrix() * camera->getViewMatrix();

	// Vertex shader
	GLuint CameraRight_worldspace_ID = glGetUniformLocation(particleShader->getProgramID(), "CameraRight_worldspace");
	GLuint CameraUp_worldspace_ID = glGetUniformLocation(particleShader->getProgramID(), "CameraUp_worldspace");
	GLuint ViewProjMatrixID = glGetUniformLocation(particleShader->getProgramID(), "VP");

	// fragment shader
	GLuint TextureID = glGetUniformLocation(particleShader->getProgramID(), "myTextureSampler");
	Model loader = Model();
	GLuint Texture = loader.TextureFromFile("dust.png", "Ressources/Texture");

	// Bind our texture in Texture Unit 8
	glActiveTexture(GL_TEXTURE8);
	glBindTexture(GL_TEXTURE_2D, Texture);
	glGenerateMipmap(GL_TEXTURE_2D);

	// Same as the billboards tutorial
	glUniform3f(CameraRight_worldspace_ID, camera->getViewMatrix()[0][0], camera->getViewMatrix()[1][0], camera->getViewMatrix()[2][0]);
	glUniform3f(CameraUp_worldspace_ID, camera->getViewMatrix()[0][1], camera->getViewMatrix()[1][1], camera->getViewMatrix()[2][1]);
	glUniformMatrix4fv(ViewProjMatrixID, 1, GL_FALSE, &ViewProjectionMatrix[0][0]);
	glUniform1i(TextureID, 8);

	updateBuffers();

}

void ParticleShader::cleanUp(){
	// Cleanup VBO and shader
	glDeleteBuffers(1, &particles_color_buffer);
	glDeleteBuffers(1, &particles_position_buffer);
	glDeleteBuffers(1, &vertex_buffer);
	//glDeleteTextures(1, &TextureID);
	//glDeleteVertexArrays(1, &VertexArrayID);
}
