#pragma once

#include "LoadShaders.h"
#include "../Timer.h";
#include "../Scene/Camera.h";
#include "../Scene/Model.h";

#include <stdio.h>
#include <stdlib.h>

#include "../../External/Include/glew/glew.h"
#include "../../External/Include/glfw/glfw3.h"
#include "../../External/Include/glm/glm.hpp"
#include <glm/gtx/norm.hpp>

#include<vector>
using namespace glm;

struct Particle{
	vec3 pos, speed;
	unsigned char r, g, b, a; // Color
	float size, angle, weight;
	float life; //if < 0 : dead and unused.
	float cameraDistance; // *Squared* distance to the camera. if dead : -1.0f

	bool operator<(const Particle& that) const {
		// Sort in reverse order : far particles drawn first.
		return this->cameraDistance > that.cameraDistance;
	}
};

const static int MaxParticles = 30;

class ParticleShader{

public:
	ParticleShader(Camera *camera, LoadShaders *particleShader);
	~ParticleShader();

	Particle particlesContainer[MaxParticles];
	int particlesPerFrame = 0;
	LoadShaders *particleShader = NULL;

	void setUpParticles(Camera *camera, Mouse *mouse, Timer *timer, int particlesPerFrame);
	void updateBuffers();
	void renderParticles();
	void setParticlesPerFrame(int newParticlesPerFrame);
	void loadParticlesToGpu();

	void cleanUp();


private:
	
	GLuint VertexArrayID = 0;
	Timer *timer = NULL;
	Camera *camera = NULL;
	Mouse *mouse = NULL;
	int lastUsedParticle = 0;
	int particlesCount = 0;

	GLuint vertex_buffer = 0;
	GLuint particles_position_buffer = 0;
	GLuint particles_color_buffer = 0;

	void createBuffers();
	int findUnusedParticle();
	void sortParticles();
};
