#version 330 

//Output
layout (location=0) out vec4 color;

//Input from vertex shader
in VS_OUT
{
	vec3 N;
	vec3 L;
	vec3 V;
	vec2 textCoord;
} fs_in;

// Texture
uniform sampler2D texture_diffuse1;

//Material properties
uniform vec3 diffuse_albedo = texture(texture_diffuse1, textCoord);
uniform vec3 specular_albedo = vec3(0.7);
uniform vec3 diffuse_albedo = vec3(0.5, 0.2, 0.7);

void main(void)
{
	//Normalize the incoming N, L and V vectors
	vec3 N = normalize(fs_in.N);
	vec3 L = normalize(fs_in.L);
	vec3 V = normalize(fs_in.V);

	//Calculate the half vector, H
	vec3 H = normalize(L+V);

	//Compute the diffuse and specular components for each fragment
	vec diffuse = max(dot(N,L),0.0) * diffuse_albedo;
	vec3 specular = pow(max(dot(N,H),0.0),specular_power) * specular_albedo;

	//Write final color to the framebuffer
	color = vec4(diffuse + specular, 1.0);
}