#include "WaterShader.h"

WaterShader::WaterShader(LoadShaders *waterShader, int systemWidth, int systemHeight, Timer *timer, Camera *camera, Lighting *light)
{
	this->systemWidth = systemWidth;
	this->systemHeight = systemHeight;
	this->waterShader = waterShader;
	this->timer = timer;
	this->camera = camera;
	this->light = light;

	initialiseReflectionFrameBuffer();
	initialiseRefractionFrameBuffer();
	CreateDuDvTexture();
	CreateNormalTexture();
	CreateLightParameters();
}

WaterShader::~WaterShader()
{
}

void WaterShader::cleanUp() {//call when closing the game
/*	glDeleteFramebuffers(1, &reflectionFrameBuffer);
	glDeleteTextures(1, reflectionTexture);
	glDeleteRenderbuffers(1, reflectionDepthBuffer);

	glDeleteFramebuffers(1, refractionFrameBuffer);
	glDeleteTextures(1, refractionTexture);
	glDeleteTextures(1, refractionDepthTexture);
	*/
	
}

void WaterShader::bindReflectionFrameBuffer() {//call before rendering to this FBO
	bindFrameBuffer(reflectionFrameBuffer, REFLECTION_WIDTH, REFLECTION_HEIGHT);
	glDrawBuffer(GL_COLOR_ATTACHMENT0);
	//bindFrameBuffer(*reflectionFrameBuffer, systemWidth, systemHeight);
}

void WaterShader::bindRefractionFrameBuffer() {//call before rendering to this FBO
	bindFrameBuffer(refractionFrameBuffer, REFRACTION_WIDTH, REFRACTION_HEIGHT);
	glDrawBuffer(GL_COLOR_ATTACHMENT0);
	//bindFrameBuffer(*refractionFrameBuffer,  systemWidth, systemHeight);
}

void WaterShader::unbindCurrentFrameBuffer() {//call to switch to default frame buffer
	glBindFramebuffer(GL_FRAMEBUFFER, 0);
	glDisable(GL_BLEND);
	glViewport(0, 0, systemWidth, systemHeight);
}

GLint WaterShader::getReflectionTexture() {//get the resulting texture
	return reflectionTexture;
}

GLint WaterShader::getRefractionTexture() {//get the resulting texture
	return refractionTexture;
}

GLint WaterShader::getRefractionDepthTexture(){//get the resulting depth texture
	return refractionDepthTexture;
}

void WaterShader::initialiseReflectionFrameBuffer() {
	reflectionFrameBuffer = createFrameBuffer();
	
	reflectionTexture = createTextureAttachment(REFLECTION_WIDTH, REFLECTION_HEIGHT);

	reflectionDepthBuffer = createDepthBufferAttachment(REFLECTION_WIDTH, REFLECTION_HEIGHT);

	unbindCurrentFrameBuffer();
}

void WaterShader::initialiseRefractionFrameBuffer() {
	
	refractionFrameBuffer = createFrameBuffer();

	refractionTexture = createTextureAttachment(REFRACTION_WIDTH, REFRACTION_HEIGHT);

	refractionDepthTexture = createDepthTextureAttachment(REFRACTION_WIDTH, REFRACTION_HEIGHT);

	unbindCurrentFrameBuffer();
}

void WaterShader::bindFrameBuffer(GLuint frameBuffer, int width, int height){
	glBindTexture(GL_TEXTURE_2D, 0);//To make sure the texture isn't bound

	glBindFramebuffer(GL_FRAMEBUFFER, frameBuffer);

	//indicate that we will always render to color attachment 0
	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) == GL_FRAMEBUFFER_UNDEFINED){
		std::cout << "ERROR: FRAMEBUFFER: " << "GL_FRAMEBUFFER_UNDEFINED" << std::endl;
	}
	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) == GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT){
		std::cout << "ERROR: FRAMEBUFFER: " << "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT" << std::endl;
	}
	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) == GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT){
		std::cout << "ERROR: FRAMEBUFFER: " << "GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT" << std::endl;
	}
	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) == GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER){
		std::cout << "ERROR: FRAMEBUFFER: " << "GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER" << std::endl;
	}
	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) == GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER){
		std::cout << "ERROR: FRAMEBUFFER: " << "GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER" << std::endl;
	}
	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) == GL_FRAMEBUFFER_UNSUPPORTED){
		std::cout << "ERROR: FRAMEBUFFER: " << "GL_FRAMEBUFFER_UNSUPPORTED" << std::endl;
	}
	glViewport(0, 0, width, height);
}

GLuint WaterShader::createFrameBuffer() {
	GLuint frameBuffer = 0;
	glGenFramebuffers(1, &frameBuffer);
	//generate name for frame buffer
	//glBindFramebuffer(GL_FRAMEBUFFER, frameBuffer);
	glBindFramebuffer(GL_FRAMEBUFFER, frameBuffer);
	//create the framebuffer
	glDrawBuffer(GL_COLOR_ATTACHMENT0);
	
	return frameBuffer;
}

GLint WaterShader::createTextureAttachment(int width, int height) {
	GLuint texture = 0;
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height,
		0, GL_RGB, GL_UNSIGNED_BYTE, NULL);
	
	glBindTexture(GL_TEXTURE_2D, 0); //new

	//attach texture to fbo
	glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
		texture, 0);
	return texture;
}

GLint WaterShader::createDepthTextureAttachment(int width, int height){
	GLuint texture = 0;
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT32, width, height,
		0, GL_DEPTH_COMPONENT, GL_FLOAT, NULL);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glFramebufferTexture(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT,
		texture, 0);
	return texture;
}

GLint WaterShader::createDepthBufferAttachment(int width, int height) {
	GLuint depthBuffer = 0;
	glGenRenderbuffers(1, &depthBuffer);
	glBindRenderbuffer(GL_RENDERBUFFER, depthBuffer);
	glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, width,
		height);

	glBindRenderbuffer(GL_RENDERBUFFER, 0);//new

	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT,
		GL_RENDERBUFFER, depthBuffer);
	return depthBuffer;
}

void WaterShader::CreateNormalTexture() {
	Model loader = Model();
	NormalTexture = loader.TextureFromFile("normalmap.jpg", "Ressources/Model/water");
	std::cout << "CreateNormalTexture finished" << std::endl;
}

GLint WaterShader::getNormalTexture() {
	return NormalTexture;
}

void WaterShader::CreateDuDvTexture() {
	Model loader = Model();
	duDvTexture = loader.TextureFromFile("dudvmap.jpg", "Ressources/Model/water");
	std::cout << "CreateDuDvTexture finished" << std::endl;
}

void WaterShader::CreateLightParameters() {
	lightColor = light->getAmbientLight(); // Ambient Light
	lightLocation = light->getLightDirection();
}

GLint WaterShader::getDuDvTexture() {
	return duDvTexture;
}

float WaterShader::moveWater() {
	float time = timer->getDeltaT();//fmod((glfwGetTime()*100),0.01f);//timer.getGemplayTime();
	moveWaterFactor += waveSpeed * time;
	moveWaterFactor = fmod(moveWaterFactor, 1.0f); // modulo 1
	return moveWaterFactor;
}

vec3 WaterShader::getCameraLocation() {
	cameraLocation = camera->getPosition();

	return cameraLocation;
}

void WaterShader::setUpWater(){

	waterShader->useShader();

	//glEnableVertexAttribArray(0);
	glActiveTexture(GL_TEXTURE2);
	glBindTexture(GL_TEXTURE_2D, getRefractionTexture());
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, getReflectionTexture());
	glActiveTexture(GL_TEXTURE3);
	glBindTexture(GL_TEXTURE_2D, getDuDvTexture());
	glActiveTexture(GL_TEXTURE6);
	glBindTexture(GL_TEXTURE_2D, getNormalTexture());
	glActiveTexture(GL_TEXTURE7);
	glBindTexture(GL_TEXTURE_2D, getRefractionDepthTexture());

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	GLint reflectionTexture = glGetUniformLocation(waterShader->getProgramID(), "reflectionTexture");
	GLint refractionTexture = glGetUniformLocation(waterShader->getProgramID(), "refractionTexture");
	GLint dudvMap = glGetUniformLocation(waterShader->getProgramID(), "dudvMap");
	GLint normalMap = glGetUniformLocation(waterShader->getProgramID(), "normalMap");
	GLint refractionDepthTexture = glGetUniformLocation(waterShader->getProgramID(), "depthMap");

	GLfloat positionLight = glGetUniformLocation(waterShader->getProgramID(), "lightPosition");
	GLfloat colorLight = glGetUniformLocation(waterShader->getProgramID(), "lightColor");

	GLfloat moveFactor = glGetUniformLocation(waterShader->getProgramID(), "moveFactor");
	GLfloat cameraPosition = glGetUniformLocation(waterShader->getProgramID(), "cameraPosition");

	glUniform1i(reflectionTexture, 1);
	glUniform1i(refractionTexture, 2);
	glUniform1i(dudvMap, 3);
	glUniform1i(normalMap, 6);
	glUniform1i(refractionDepthTexture, 7);

	glUniform1f(moveFactor, moveWater());
	glUniform3f(cameraPosition, getCameraLocation().x, getCameraLocation().y, getCameraLocation().z);
	glUniform3f(positionLight, lightLocation.x, lightLocation.y, lightLocation.z);
	glUniform3f(colorLight, lightColor.x, lightColor.y, lightColor.z);
}