#pragma once
#include <math.h>
#include "../Scene/Water.h"
#include "LoadShaders.h"
#include "../Scene/Model.h"
#include "../Timer.h"
#include "../Scene/Camera.h"
#include "../Scene/Lighting.h"

#include <stdio.h>
#include <stdlib.h>

#include "../../External/Include/glew/glew.h"
#include "../../External/Include/glfw/glfw3.h"
#include "../../External/Include/glm/glm.hpp"

#include<vector>
using namespace glm;

class WaterShader{

public:
	WaterShader(LoadShaders *waterShader, int systemWidth, int systemHeight, Timer *timer, Camera *camera, Lighting *light);
	~WaterShader();

	LoadShaders *waterShader = NULL;

	int systemWidth = 0;
	int systemHeight = 0;

	void cleanUp();
	void bindReflectionFrameBuffer();
	void bindRefractionFrameBuffer();

	void unbindCurrentFrameBuffer();

	void setUpWater();

	GLint getReflectionTexture();
	GLint getRefractionTexture();
	GLint getRefractionDepthTexture();
	GLint getDuDvTexture();
	GLint getNormalTexture();

	
private:
	const int REFLECTION_WIDTH = 320;
	const int REFLECTION_HEIGHT = 180;

	const int REFRACTION_WIDTH = 1280;
	const int REFRACTION_HEIGHT = 720;

	GLint reflectionFrameBuffer = 0;
	GLint reflectionTexture = 0;
	GLint reflectionDepthBuffer = 0;

	GLint refractionFrameBuffer = 0;
	GLint refractionTexture = 0;
	GLint refractionDepthTexture = 0;

	GLint NormalTexture = 0;

	GLint duDvTexture = 0;
	float moveWaterFactor = 0.0f; // move effect of the water
	float waveSpeed = 0.0003f; 
	Timer *timer = NULL;

	Lighting *light = NULL;
	vec3 lightColor = vec3();
	vec3 lightLocation = vec3();

	Camera *camera = NULL;
	vec3 cameraLocation = vec3();

	float moveWater();
	vec3 getCameraLocation();

	void initialiseReflectionFrameBuffer();
	void initialiseRefractionFrameBuffer();

	GLuint createFrameBuffer();
	void bindFrameBuffer(GLuint frameBuffer, int width, int height);

	GLint createTextureAttachment(int width, int height);
	GLint createDepthTextureAttachment(int width, int height);
	GLint createDepthBufferAttachment(int width, int height);
	
	void CreateDuDvTexture();
	void CreateNormalTexture();
	void CreateLightParameters();

};